#!/usr/bin/python

## groupProperties - event handling code for userconf's group properties
## Copyright (C) 2001 Red Hat, Inc.
## Copyright (C) 2001 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Brent Fox


import signal
import gtk
import gnome.ui
import libglade
from gtk import *
from gnome.ui import *
import string
import os
import checklist
import mainWindow
import libuser

class groupProperties:
    def __init__(self, parent, user_clist, group_clist, xml):
        self.parent = parent
        self.user_clist = user_clist
        self.group_clist = group_clist

        self.groupWin = xml.get_widget('groupProperties')
        self.groupWin.close_hides(TRUE)
        self.groupWinGroupName = xml.get_widget('groupNameEntry')
        self.groupNotebook = xml.get_widget('groupNotebook')
        self.checklistScroll = xml.get_widget('userListScroll')

        self.userChecklist = checklist.CheckList(2)
        self.userChecklist.connect ("button_press_event", self.mouse_button_press)
        self.userChecklist.connect ("key_press_event", self.key_press_cb)
        self.userChecklist.show()
        self.checklistScroll.add(self.userChecklist)

        xml.signal_autoconnect (
            { "on_groupProperties_cancel_button_clicked" : self.on_groupProperties_cancel_button_clicked,
              "on_groupProperties_ok_button_clicked" : self.on_groupProperties_ok_button_clicked,
              "on_groupProperties_apply_button_clicked" : self.on_groupProperties_apply_button_clicked,
              })
             
    def groupWinReset(self):
        self.groupNotebook.set_page(0)
        self.userChecklist.clear()

    def showGroupProperties(self, groupEnt, row):
        self.groupEnt = groupEnt
        self.row = row
        self.groupWinReset()

        cn = self.groupEnt.get(libuser.GROUPNAME)[0]
        self.groupWinGroupName.set_text(cn)

        self.fill_users_list(self.groupEnt)
        self.groupWin.show_all()

    def on_groupProperties_cancel_button_clicked(self, *args):
        self.groupWinReset()
        self.groupWin.hide()

    def on_groupProperties_ok_button_clicked(self, *args):
        self.on_groupProperties_apply_button_clicked()
        self.groupWinReset()
        self.groupWin.hide()
        
    def on_groupProperties_apply_button_clicked(self, *args):
        newGroupName = self.groupWinGroupName.get_text()
        currentGroupName = self.groupEnt.get(libuser.GROUPNAME)[0]

        if newGroupName != currentGroupName:
            self.groupEnt.set(libuser.GROUPNAME, newGroupName)
            self.group_clist.set_text(self.row, 0, newGroupName)

            self.group_clist.set_row_data(self.row, self.groupEnt)
        
        user_list = []
        
        group_gidNumber = self.groupEnt.get(libuser.GIDNUMBER)[0]
        members = self.groupEnt.get(libuser.MEMBERUID)
        if not members:
            members = []

        for i in range(len(self.users)):
            val, user, None = self.userChecklist.get_row_data(i)

            if val:
                if user in members:
                    user_list.insert(0, user)
                else:
                    # don't add the user to the group if it's her primary group
                    try:
                        userEnt = self.parent.user_dict[user]
                    except:
                        userEnt = self.parent.ADMIN.lookupUserByName(user)
                    user_gidNumber = userEnt.get(libuser.GIDNUMBER)[0]
                    if user_gidNumber != group_gidNumber:
                        user_list.append(user)
            else:  
                if user in members:
                    members.remove(user)

        self.groupEnt.set(libuser.MEMBERUID, user_list)
        self.parent.ADMIN.modifyGroup(self.groupEnt)

        self.parent.refresh_users_and_groups(user_list + [currentGroupName] + [newGroupName])

    def toggle_row (self, row):
        (val, row_data, name) = self.userChecklist.get_row_data(row)
        val = not val
        self.userChecklist.set_row_data(row, (val, row_data, name))
        self.userChecklist._update_row(row)

        if self.userChecklist.toggled_func != None:
            self.userChecklist.toggled_func(val)

    def mouse_button_press(self, clist, event):
        row, col = self.userChecklist.get_selection_info(event.x, event.y)
        if row != None:
            self.toggle_row(row)

    def key_press_cb (self, clist, event):
        if event.keyval == ord(" ") and self.userChecklist.focus_row != -1:
            self.toggle_row(self.userChecklist.focus_row)

    def fill_users_list (self, groupEnt):
        self.users = self.parent.ADMIN.enumerateUsers()
        self.users.sort()
        row = 0
        group = groupEnt.get(libuser.GROUPNAME)[0]
        members = self.parent.ADMIN.enumerateUsersByGroup(group)
        if not members:
            members = []

        for user in self.users:
            if user in members:            
                row = self.userChecklist.append_row((user, user), TRUE, user)
                self.userChecklist.set_row_data(row, (1, user, user))
            else:
                row = self.userChecklist.append_row((user, user), FALSE, user)
                self.userChecklist.set_row_data(row, (0, user, user))

