#!/usr/bin/python

## mainWindow - main interface window for userconf
## Copyright (C) 2001 Red Hat, Inc.
## Copyright (C) 2001 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Brent Fox

import signal
import gtk
import gettext
import gnome.ui
import libglade
import GTK
from gtk import *
from gnome.ui import *
import gnome.help
import string
import sys
import time
import libuser

import groupWindow
import userWindow
import userProperties
import groupProperties

domain = 'redhat-config-users'
gettext.bindtextdomain(domain, '/usr/share/locale')

def _(string):
    return gettext.dgettext(domain, string)

def service_pending_events():
    while gtk.events_pending():
        gtk.mainiteration()

def admin_prompt_callback(prompts, title):
    dialog = GnomeDialog(title = title, b1 = gnome.ui.STOCK_BUTTON_OK, b2 = gnome.ui.STOCK_BUTTON_CANCEL)
    dialog.set_position(GTK.WIN_POS_CENTER)

    table = gtk.GtkTable(rows = len(prompts), cols = 2)
    dialog.vbox.pack_start(table)

    table.set_row_spacings(4)
    table.set_col_spacings(4)

    ret_list = []
    for i in range(len(prompts)):
        prompt = prompts[i]

        label = gtk.GtkLabel(prompt.prompt)
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0, 1, i, i + 1)

        entry = gtk.GtkEntry()
        entry.set_visibility(prompt.visible)
        if prompt.default_value:
            entry.set_text(prompt.default_value)
        table.attach(entry, 1, 2, i, i + 1)

        ret_list.append((prompt, entry))
        
    table.show_all()

    i = dialog.run_and_close()
    if i == 0:
        for (prompt, entry) in ret_list:
            prompt.value = entry.get_text()
        return TRUE

    sys.exit(1)


xml = libglade.GladeXML ("/usr/share/redhat-config-users/redhat-config-users.glade", domain="redhat-config-users")
#xml = libglade.GladeXML ("./redhat-config-users.glade", domain="redhat-config-users")
busy_cursor = gtk.cursor_new(GDK.WATCH)
ready_cursor = gtk.cursor_new(GDK.LEFT_PTR)

class mainWindow:

    def version():
        # substituted to the real version by the Makefile at installation time.
        return "0.9.2"

    def destroy (self, args):
        gtk.mainquit()

    def busy(self):
        self.toplevel.set_sensitive(FALSE)
        window = self.toplevel.get_window()
        window.set_cursor(busy_cursor)
        pass

    def ready(self):
        window = self.toplevel.get_window()
        window.set_cursor(ready_cursor)
        self.toplevel.set_sensitive(TRUE)
        pass

    def get_user_list (self, filter):
        if filter == "":
            users = self.ADMIN.enumerateUsers()
        else:
            users = self.ADMIN.enumerateUsers(filter)
        users.sort()
        return users

    def get_group_list(self, filter):
        if filter == "":
            groups = self.ADMIN.enumerateGroups()
        else:
            groups = self.ADMIN.enumerateGroups(filter)
        groups.sort()
        return groups

    def refresh_users(self):
        self.busy()
        self.user_dict = {}
        self.gid_dict = {}

        # pull up information about users and store the objects in
	# a dictionary keyed by user name
        i = 0
        for user in self.get_user_list(self.get_filter_data()):
            i = i + 1
            if i >= self.service_interval:
                service_pending_events()
                i = 0
            userEnt = self.ADMIN.lookupUserByName(user)
            self.user_dict[user] = userEnt

            # try to get a name to associate with the user's primary gid,
            # and attempt to minimize lookups by caching answers
            gidNumber = userEnt.get(libuser.GIDNUMBER)[0]
            try:
                groupEnt = self.gid_dict[int(gidNumber)]
            except:
                try:
                    groupEnt = self.ADMIN.lookupGroupById(int(gidNumber))
                    self.gid_dict[int(gidNumber)] = groupEnt
                except:
                    pass

        self.ready()

    def refresh_groups(self):
        self.busy()
        self.group_dict = {}
        self.member_dict = {}

        i = 0
        for group in self.get_group_list(self.get_filter_data()):
            i = i + 1
            if i >= self.service_interval:
                service_pending_events()
                i = 0
            groupEnt = self.ADMIN.lookupGroupByName(group)
            self.group_dict[group] = groupEnt

            members = self.ADMIN.enumerateUsersByGroup(group)
            if not members:
                members = []
            members.sort()
            self.member_dict[group] = members

        self.ready()

    def refresh_users_and_groups(self, names):
        self.busy()
        updated_names = {}
        for name in names:
            # If we've already updated this user/group, skip it.
            if updated_names.has_key(name):
                continue
            updated_names[name] = name
            service_pending_events()

            userEnt = self.ADMIN.lookupUserByName(name)
            if userEnt:
                self.user_dict[name] = userEnt
                gidNumber = userEnt.get(libuser.GIDNUMBER)[0]
                try:
                    groupEnt = self.gid_dict[int(gidNumber)]
                except:
                    try:
                        groupEnt = self.ADMIN.lookupGroupById(int(gidNumber))
                        self.gid_dict[int(gidNumber)] = groupEnt
                    except:
                        pass
            else:
                try:
                    del(self.user_dict[name])
                except:
                    pass

            groupEnt = self.ADMIN.lookupGroupByName(name)
            if groupEnt:
                self.group_dict[name] = groupEnt
                self.member_dict[name] = self.ADMIN.enumerateUsersByGroup(name)
                gidNumber = groupEnt.get(libuser.GIDNUMBER)
                gidNumber = gidNumber[0]
                self.gid_dict[int(gidNumber)] = groupEnt
            else:
                try:
                    del(self.group_dict[name])
                    del(self.member_dict[name])
                except:
                    pass
            # print "update", name

        users = self.get_user_list(self.get_filter_data())
        for user in self.user_dict.keys():
            if user not in users:
                del self.user_dict[user]

        groups = self.get_group_list(self.get_filter_data())
        for group in self.group_dict.keys():
            if group not in groups:
                del self.group_dict[group]

        self.populate_clists()
        self.ready()

    def refresh(self):
        self.busy()
        self.refresh_users()
        self.refresh_groups()
        self.populate_clists()
        self.ready()

    def populate_user_clist(self):
        self.busy()
        self.user_clist.clear()
        self.user_clist.freeze()
        for user in self.user_dict.keys():
            userEnt = self.user_dict[user]
            uid = userEnt.get(libuser.USERNAME)[0]
            uidNumber = userEnt.get(libuser.UIDNUMBER)[0]
            gidNumber = userEnt.get(libuser.GIDNUMBER)[0]
            if userEnt.get(libuser.GECOS):
                gecos = userEnt.get(libuser.GECOS)[0]
            else:
                gecos = uid
            if userEnt.get(libuser.HOMEDIRECTORY):
                homeDir = userEnt.get(libuser.HOMEDIRECTORY)[0]
            else:
                homeDir = ''
            if userEnt.get(libuser.LOGINSHELL):
                shell = userEnt.get(libuser.LOGINSHELL)[0]
            else:
                shell = ''

            try:
                groupEnt = self.gid_dict[int(gidNumber)]
                groupName = groupEnt.get(libuser.GROUPNAME)[0]
            except:
                groupName = gidNumber

            #display users with UIDs > 499
            if int(uidNumber) > -1:
                row = self.user_clist.append([uid, uidNumber, groupName, gecos, shell, homeDir, "", ""])
                self.user_clist.set_row_data(row, userEnt)
        self.user_clist.thaw()
        self.ready()

    def populate_group_clist(self):
        self.busy()
        self.group_clist.clear()
        self.group_clist.freeze()
        for group in self.group_dict.keys():
            groupEnt = self.group_dict[group]
            cn = groupEnt.get(libuser.GROUPNAME)[0]
            gid = groupEnt.get(libuser.GIDNUMBER)[0]
            members = self.member_dict[group]
            if not members:
                members = []

            #concatenate the list of members in the group into a comma separated list
            memberlist = string.join(members, ", ")
            #display groups with UIDs > 499
            if int(gid) > -1:
                row = self.group_clist.append([cn, gid, memberlist])
                self.group_clist.set_row_data(row, groupEnt)
        self.group_clist.thaw()
        self.ready()

    def populate_clists(self):
        self.populate_user_clist()
        self.populate_group_clist()

    def __init__(self):
        self.toplevel = xml.get_widget ('mainWindow')
        self.toplevel.connect("destroy", self.destroy)
        self.busy()

        self.ADMIN = libuser.admin(prompt=admin_prompt_callback, prompt_data=(_('Logon Information Required'),))

        self.user_clist = xml.get_widget('user_clist')
        self.user_clist.set_column_visibility(1, FALSE)
#        self.user_clist.set_column_visibility(2, FALSE)
        self.user_clist.set_column_visibility(6, FALSE)
        self.user_clist.set_column_visibility(7, FALSE)
        self.user_clist.set_auto_sort(TRUE)
        self.group_clist = xml.get_widget('group_clist')
        self.group_clist.set_auto_sort(TRUE)
        self.notebook = xml.get_widget('notebook1')

#        self.statusbar = xml.get_widget ('statusbar1')
#        num = self.statusbar.get_context_id("")

        self.filter = xml.get_widget('filterEntry')

        self.user_dict = {}
        self.group_dict = {}
        self.gid_dict = {}
        self.member_dict = {}
        self.service_interval = 10
        self.refresh()

        self.userWin = userWindow.userWindow(self, self.user_clist, self.group_clist, xml)
        self.groupWin = groupWindow.groupWindow(self, self.user_clist, self.group_clist, xml)
        self.userProperties = userProperties.userProperties(self, self.user_clist, self.group_clist, xml)
        self.groupProperties = groupProperties.groupProperties(self, self.user_clist, self.group_clist, xml)

        self.userSelectedRow = None
        self.groupSelectedRow = None
        
        self.toplevel.show_all ()
        self.ready()

        xml.signal_autoconnect (
            { "on_help_button_clicked" : self.on_help_button_clicked,
              "on_about_activate" : self.on_about_button_clicked,
              "on_manual_activate" : self.on_manual_button_clicked,
              "on_add_user_activate" : self.on_add_user_activate,
              "on_add_user_button_clicked" : self.on_add_user_activate,
              "on_add_group_activate" : self.on_add_group_activate,
              "on_add_group_button_clicked" : self.on_add_group_activate,
              "on_properties_activate" : self.on_properties_activate,
              "on_properties_button_clicked" : self.on_properties_activate,
              "on_delete_activate" : self.on_delete_activate,
              "on_delete_button_clicked" : self.on_delete_activate,
              "on_user_clist_select_row" : self.on_user_clist_select_row,
              "on_group_clist_select_row" : self.on_group_clist_select_row,
              "on_new_user_activate" : self.on_add_user_activate,
              "on_filterButton_clicked" : self.on_filterButton_clicked,
              "on_filterEntry_activate" : self.on_filterButton_clicked,
              "on_refreshButton_clicked" : self.on_filterButton_clicked,
              "on_exit_activate"     : gtk.mainquit,
              })

        gtk.mainloop ()

    #------------Event handlers-------------#
    #---------------------------------------#

    #--------Event handlers for toplevel window-----#
    def on_about_button_clicked(self, *args):
        dlg = gnome.ui.GnomeAbout("Red Hat User Manager",
                                  "0.9.2",
                                  "Copyright (c) 2001 Red Hat, Inc.",
                                  ["Brent Fox <bfox@redhat.com>"],
                                  _("This software is distributed under the GPL."
                                  "Please Report bugs to Red Hat's Bug Tracking"
                                  "System: http://bugzilla.redhat.com/"))
        dlg.run_and_close()

    def on_help_button_clicked(self, *args):
        help_pages = ["file:///usr/share/doc/redhat-config-users-0.9.2/user-new.html",
                      "file:///usr/share/doc/redhat-config-users-0.9.2/group-new.html",]
        gnome.help.goto (help_pages [self.notebook.get_current_page ()])

    def on_manual_button_clicked(self, *args):
        gnome.help.goto ("file:///usr/share/doc/redhat-config-users-0.9.2/index.html")

    def on_exit1_activate(self, *args):
        gtk.mainquit()

    def on_add_user_activate(self, *args):
        filter = self.get_filter_data()
        self.userWin.newUserWin(filter)

    def on_add_group_activate(self, *args):
        self.groupWin.newGroupWin()

    def on_delete_activate(self, *args):
        page = self.notebook.get_current_page()
        if page == 0:
            if self.userSelectedRow != None:
                userEnt = self.user_clist.get_row_data(self.userSelectedRow)
                user = userEnt.get(libuser.USERNAME)[0]
                self.ADMIN.deleteUser(userEnt)
                try:
                    del self.user_dict[user]
                except:
                    pass
                self.user_clist.remove(self.userSelectedRow)

                need_refresh = [user]
                for group in self.group_dict.keys():
                    groupEnt = self.group_dict[group]
                    members = groupEnt.get(libuser.MEMBERUID)

                    if members:
                        if user in members:
                            members.remove(user)
                            groupEnt.set(libuser.MEMBERUID, members)
                            self.ADMIN.modifyGroup(groupEnt)
                            need_refresh.append(group)

                    self.userSelectedRow = None
                self.refresh_users_and_groups(need_refresh)

        elif page == 1:
            if self.groupSelectedRow != None:
                groupEnt = self.group_clist.get_row_data(self.groupSelectedRow)
                group = groupEnt.get(libuser.GROUPNAME)[0]
                self.ADMIN.deleteGroup(groupEnt)
                try:
                    del self.group_dict[group]
                except:
                    pass
                self.group_clist.remove(self.groupSelectedRow)
                self.groupSelectedRow = None

    def on_properties_activate(self, *args):        
        page = self.notebook.get_current_page()
        if page == 0:
            self.user_properties()
        elif page == 1:
            self.group_properties()

    def user_properties(self):
        for row in self.user_clist.selection:
            userEnt = self.user_clist.get_row_data(row)
            self.userProperties.showUserProperties(userEnt, row)

    def group_properties(self):
        for row in self.group_clist.selection:
            groupEnt = self.group_clist.get_row_data(row)
            self.groupProperties.showGroupProperties(groupEnt, row)

    def get_filter_data(self):
        filter = self.filter.get_text()
        filter = string.strip(filter)

        if len(filter) == 0:
            filter = ''
        else:
            index = string.find(filter, '*')
            length = len(filter) - 1

            if index != length:
                filter = filter + "*"
        return filter

    def on_filterButton_clicked(self, *args):
        self.refresh()

    def on_user_clist_select_row(self, *args):
        self.userSelectedRow = args[1]
        self.userSelectedCol = args[2]

    def on_group_clist_select_row(self, *args):
        self.groupSelectedRow = args[1]
        self.groupSelectedCol = args[2]
