#!/usr/bin/python

## userProperties - event handling code for userconf's user properties
## Copyright (C) 2001 Red Hat, Inc.
## Copyright (C) 2001 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Brent Fox

import exceptions
from gtk import *
from gnome.ui import *
import gnome.help
import gnome.config
import time
import string
import checklist
import math
import types
import libuser
import mainWindow

class userProperties:
    def __init__(self, parent, user_clist, group_clist, xml):
        self.parent = parent
        self.user_clist = user_clist
        self.group_clist = group_clist

        self.userWin = xml.get_widget('userProperties')
        self.userWin.close_hides(TRUE)
        self.userWinUserName = xml.get_widget('userNameEntry')
        self.userWinFullName = xml.get_widget('fullNameEntry')
        self.userWinPassword = xml.get_widget('passwordEntry')
        self.userWinConfirm = xml.get_widget('confirmEntry')
        self.userWinHomeDir = xml.get_widget('homeDirEntry')
        self.loginShellCombo = xml.get_widget('loginShellCombo')
        self.userNotebook = xml.get_widget('userNotebook')
        self.checklistScroll = xml.get_widget('groupListScroll')
        self.dayLabel = xml.get_widget('dayLabel')
        self.monthLabel = xml.get_widget('monthLabel')
        self.dateLabel = xml.get_widget('dateLabel')
        self.yearLabel = xml.get_widget('yearLabel')

        self.accountLockCheck = xml.get_widget('accountLockCheck')
        self.accountExpireCheck = xml.get_widget('accountExpireCheck')
        self.accountMonthEntry = xml.get_widget('accountMonthEntry')
        self.accountDayEntry = xml.get_widget('accountDayEntry')
        self.accountYearEntry = xml.get_widget('accountYearEntry')
        self.accountHBox = xml.get_widget('newAccountHBox')
        self.pwExpireCheck = xml.get_widget('pwExpireCheck')
        self.pwExpireTable = xml.get_widget('pwExpireTable')
        self.pwAllowEntry = xml.get_widget('pwAllowEntry')
        self.pwRequireEntry = xml.get_widget('pwRequireEntry')
        self.pwWarnEntry = xml.get_widget('pwWarnEntry')
        self.pwInactiveEntry = xml.get_widget('pwInactiveEntry')

        self.groupChecklist = checklist.CheckList(2)
        self.groupChecklist.connect ("button_press_event", self.mouse_button_press)
        self.groupChecklist.connect ("key_press_event", self.key_press_cb)
        self.groupChecklist.show()
        self.checklistScroll.add(self.groupChecklist)

        self.shells = self.parent.ADMIN.getUserShells()
        self.shells.sort()
        self.loginShellCombo.set_popdown_strings(self.shells)

        xml.signal_autoconnect (
            { "on_cancel_button_clicked" : self.on_cancel_button_clicked,
              "on_ok_button_clicked" : self.on_ok_button_clicked,
              "on_apply_button_clicked" : self.on_apply_button_clicked,
              "on_accountExpireCheck_toggled" : self.on_accountExpireCheck_toggled,
              "on_pwExpireCheck_toggled" : self.on_pwExpireCheck_toggled,
              })

    def userWinReset(self):
        self.userNotebook.set_page(0)
        self.userWinUserName.set_text('')
        self.userWinFullName.set_text('')
        self.userWinPassword.set_text('')
        self.userWinConfirm.set_text('')
        self.userWinHomeDir.set_text('')
        self.groupChecklist.clear()

        self.accountLockCheck.set_active(FALSE)
        self.accountExpireCheck.set_active(FALSE)
        self.accountMonthEntry.set_text("")
        self.accountDayEntry.set_text("")
        self.accountYearEntry.set_text("")
        self.pwExpireCheck.set_active(FALSE)
        self.pwAllowEntry.set_text('0')
        self.pwRequireEntry.set_text('0')
        self.pwWarnEntry.set_text('0') 
        self.pwInactiveEntry.set_text('0') 
        
    def showUserProperties(self, userEnt, row):
        self.userEnt = userEnt
        self.row = row
        self.userWinReset()
        uid = self.userEnt.get(libuser.USERNAME)[0]
        fn = self.userEnt.get(libuser.GECOS)[0]
        hd = self.userEnt.get(libuser.HOMEDIRECTORY)[0]
        shell = self.userEnt.get(libuser.LOGINSHELL)[0]

        self.userWinUserName.set_text(uid)
        self.userWinFullName.set_text(fn)
        self.userWinPassword.set_text('     ')
        self.userWinConfirm.set_text('     ')
        self.userWinHomeDir.set_text(hd)

        if shell not in self.shells:
            self.shells.append(shell)
            self.shells.sort()
            self.loginShellCombo.set_popdown_strings(self.shells)
        self.loginShellCombo.list.select_item(self.shells.index(shell))

        lastChange = self.userEnt.get(libuser.SHADOWLASTCHANGE)
        if lastChange:
            days = lastChange[0]
            seconds = 24 * 60 * 60
            tmp = int(days) * int(seconds)
            age = time.ctime(tmp)
            day, month, date, tm, year = string.split(age)
            self.dayLabel.set_text(day)
            self.monthLabel.set_text(month)
            self.dateLabel.set_text(date)
            self.yearLabel.set_text(year)

        expire = self.userEnt.get(libuser.SHADOWEXPIRE)
        if expire:
            try:
                if int(expire[0]) != -1:
                    self.accountExpireCheck.set_active(TRUE)
                    days = int(self.userEnt.get(libuser.SHADOWEXPIRE)[0])
                    tmp = days * int(seconds)
                    age = time.localtime(tmp)
                    year = str(age[0])
                    month = str(age[1])
                    day = str(age[2])
                    self.accountMonthEntry.set_text(month)
                    self.accountDayEntry.set_text(day)
                    self.accountYearEntry.set_text(year)
            except:
                    self.accountExpireCheck.set_active(FALSE)

        
        

        min = self.userEnt.get(libuser.SHADOWMIN)
        if min:
            try:
                if int(min[0]) != -1:
                    self.pwExpireCheck.set_active(TRUE)
                    self.pwAllowEntry.set_text(min[0])
            except:
                self.pwExpireCheck.set_active(FALSE)

        warning = self.userEnt.get(libuser.SHADOWWARNING)
        if warning:
            try:
                if int(warning[0]) != -1:
                    self.pwExpireCheck.set_active(TRUE)
                    self.pwWarnEntry.set_text(warning[0])
            except:
                self.pwExpireCheck.set_active(FALSE)

        inactive = self.userEnt.get(libuser.SHADOWINACTIVE)
        if inactive:
            try:
                if int(inactive[0]) != -1:
                    self.pwExpireCheck.set_active(TRUE)
                    self.pwInactiveEntry.set_text(inactive[0])
            except:
                self.pwExpireCheck.set_active(FALSE)

        if self.parent.ADMIN.userIsLocked(self.userEnt) == 1:
            self.accountLockCheck.set_active(TRUE)

        max = self.userEnt.get(libuser.SHADOWMAX)
        if max:
            try:
                if int(max[0]) != 99999:
                    self.pwExpireCheck.set_active(TRUE)
                    self.pwRequireEntry.set_text(max[0])
            except:
                self.pwExpireCheck.set_active(FALSE)

        self.fill_groups_list(self.userEnt)
        self.userWin.show_all()

    def on_cancel_button_clicked(self, *args):
        self.userWinReset()
        self.userWin.hide()

    def on_ok_button_clicked(self, *args):
        val = self.on_apply_button_clicked()
        if val == 0:
            self.userWinReset()
            self.userWin.hide()

    def on_apply_button_clicked(self, *args):
        uid = self.userWinUserName.get_text()
        gecos = self.userWinFullName.get_text()
        pw = self.userWinPassword.get_text()
        confirm = self.userWinConfirm.get_text()

        hd = self.userWinHomeDir.get_text()
        shell = self.loginShellCombo.entry.get_text()
        primaryGroup = self.userEnt.get(libuser.GIDNUMBER)[0]

        if pw == confirm == '     ':
            pass
        elif pw == confirm and len (pw) >= 6:
            self.parent.ADMIN.setpassUser(self.userEnt, pw)
        else:
            if not pw and not confirm:
                dlg = GnomeMessageBox("Please enter password for the user.",
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()
                return
            elif len (pw) < 6:
                dlg = GnomeMessageBox("Password is too short.",
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()
                return
            else:
                dlg = GnomeMessageBox("Passwords do not match.",
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()                
                return

        self.userEnt.set(libuser.USERNAME, uid)
        self.userEnt.set(libuser.GECOS, gecos)
        self.userEnt.set(libuser.HOMEDIRECTORY, hd)
        self.userEnt.set(libuser.LOGINSHELL, shell)

        group_list = []
        need_refresh = [uid]
        members = []

        for i in range(len(self.groups)):
            val, group, None = self.groupChecklist.get_row_data(i)

            try:
                groupEnt = self.parent.group_dict[group]
            except:
                groupEnt = self.parent.ADMIN.lookupGroupByName(group)
            gid = groupEnt.get(libuser.GIDNUMBER)[0]
            members = groupEnt.get(libuser.MEMBERUID)
            if not members:
                members = []

            if val:
                try:
                   index = members.index(uid)
                except:
                   index = -1

                if index == -1:
                    # the user is not in the group, but should be
                    need_refresh.append(group)
                    if primaryGroup != gid:
                        members.append(uid)
                        groupEnt.set(libuser.MEMBERUID, members)
                        self.parent.ADMIN.modifyGroup(groupEnt)
            else:
                try:
                   index = members.index(uid)
                except:
                   index = -1
                if index >= 0:
                    # the user is not supposed to be in the group, but is
                    need_refresh.append(group)
                    members.remove(uid)
                    groupEnt.set(libuser.MEMBERUID, members)
                    self.parent.ADMIN.modifyGroup(groupEnt)

        if self.accountExpireCheck.get_active():
            year = int(self.accountYearEntry.get_text())
            month = int(self.accountMonthEntry.get_text())
            day = int(self.accountDayEntry.get_text())

            timetuple = [ year, month, day, 1, 1, 1, 1, 1, 1]
            tmp = time.mktime(timetuple)
            seconds = 24 * 60 * 60
            daysTillExpire = tmp / seconds
            fraction, integer = math.modf(daysTillExpire)

            if fraction == 0.0:
                daysTillExpire = str(int(tmp/seconds))
            else:
                daysTillExpire = str(int((tmp/seconds)+1))

            self.userEnt.set(libuser.SHADOWEXPIRE, daysTillExpire)
        else:
            self.userEnt.set(libuser.SHADOWEXPIRE, "-1")

        if self.pwExpireCheck.get_active():
            allowed = self.pwAllowEntry.get_text()
            required = self.pwRequireEntry.get_text()
            warning = self.pwWarnEntry.get_text()
            inactive = self.pwInactiveEntry.get_text()

            self.userEnt.set(libuser.SHADOWMIN, allowed)
            self.userEnt.set(libuser.SHADOWMAX, required)
            self.userEnt.set(libuser.SHADOWWARNING, warning)
            self.userEnt.set(libuser.SHADOWINACTIVE, inactive)
            
        else:
            self.userEnt.set(libuser.SHADOWMIN, '-1')
            self.userEnt.set(libuser.SHADOWMAX, '99999')
            self.userEnt.set(libuser.SHADOWWARNING, '-1')
            self.userEnt.set(libuser.SHADOWINACTIVE, '-1')

        self.parent.ADMIN.modifyUser(self.userEnt)

        #Enable this when a bug in libuser is fixed
        if self.accountLockCheck.get_active():
            if self.parent.ADMIN.userIsLocked(self.userEnt) == 0:
                self.parent.ADMIN.lockUser(self.userEnt)
        else:
            if self.parent.ADMIN.userIsLocked(self.userEnt) == 1:
                self.parent.ADMIN.unlockUser(self.userEnt)


        self.parent.refresh_users_and_groups(need_refresh)
        return 0

    def toggle_row (self, row):
        (val, row_data, name) = self.groupChecklist.get_row_data(row)
        val = not val
        self.groupChecklist.set_row_data(row, (val, row_data, name))
        self.groupChecklist._update_row(row)

        if self.groupChecklist.toggled_func != None:
            self.groupChecklist.toggled_func(val)

    def mouse_button_press(self, clist, event):
        try:
            row, col = self.groupChecklist.get_selection_info(event.x, event.y)
        except:
            pass
        if row != None:
            self.toggle_row(row)

    def on_accountExpireCheck_toggled(self, *args):
        self.accountHBox.set_sensitive(self.accountExpireCheck.get_active())

    def on_pwExpireCheck_toggled(self, *args):
        self.pwExpireTable.set_sensitive(self.pwExpireCheck.get_active())

    def key_press_cb (self, clist, event):
        if event.keyval == ord(" ") and self.groupChecklist.focus_row != -1:
            self.toggle_row(self.groupChecklist.focus_row)

    def fill_groups_list (self, userEnt):
        self.groups = self.parent.ADMIN.enumerateGroups()
        self.groups.sort()
        row = 0
        uid = userEnt.get(libuser.USERNAME)[0]
        usergroups = self.parent.ADMIN.enumerateGroupsByUser(uid)

        for group in self.groups:
            if group in usergroups:
                row = self.groupChecklist.append_row((group, group), TRUE, group)
                self.groupChecklist.set_row_data(row, (1, group, group))
            else:
                row = self.groupChecklist.append_row((group, group), FALSE, group)
                self.groupChecklist.set_row_data(row, (0, group, group))
