#!/usr/bin/python

## userWindow - event handling code for userconf's user window
## Copyright (C) 2001 Red Hat, Inc.
## Copyright (C) 2001 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Brent Fox

from gtk import *
from gnome.ui import *
import string
import gnome.help
import gnome.config
import libuser
import checklist
import mainWindow
import groupDialog

##
## I18N
## 
import gettext
gettext.bindtextdomain ("redhat-config-users", "/usr/share/locale")
gettext.textdomain ("redhat-config-users")
_=gettext.gettext

class userWindow:

    def __init__(self, parent, user_clist, group_clist, xml):
        self.parent = parent
        self.user_clist = user_clist
        self.group_clist = group_clist
        
        self.userWin = xml.get_widget('newUserWindow')
        self.userWin.close_hides(TRUE)
        self.newUserNotebook = xml.get_widget('newUserNotebook')
        self.userWinUserName = xml.get_widget('newUserNameEntry')
        self.userWinFullName = xml.get_widget('newFullNameEntry')
        self.userWinPassword = xml.get_widget('newPasswordEntry')
        self.userWinConfirm = xml.get_widget('newConfirmEntry')
        self.userWinHomeDir = xml.get_widget('newHomeDirEntry')
        self.loginShellCombo = xml.get_widget('newLoginShellCombo')
        self.newGroupCheck = xml.get_widget('newGroupCheck')
        self.passwordLabel = xml.get_widget('passwordLabel')
        self.homeDirHbox = xml.get_widget('homeDirHbox')
        self.homeDirCheck = xml.get_widget('homeDirCheck')

#        self.checklistScroll = xml.get_widget('newGroupScroll')
        
#        self.groupChecklist = checklist.CheckList(2)
#        self.groupChecklist.connect ("button_press_event", self.mouse_button_press)
#        self.groupChecklist.connect ("key_press_event", self.key_press_cb)
#        self.groupChecklist.show()
#        self.checklistScroll.add(self.groupChecklist)

        self.shells = self.parent.ADMIN.getUserShells()
        self.shells.sort()
        self.loginShellCombo.set_popdown_strings(self.shells)
        default = '/bin/bash'
        if default in self.shells:
            self.loginShellCombo.list.select_item(self.shells.index(default))

        self.groupDialog = groupDialog.groupDialog(self, xml)

        xml.signal_autoconnect (
            { "on_userWin_cancel_button_clicked" : self.on_userWin_cancel_button_clicked,
              "on_userWin_ok_button_clicked" : self.on_userWin_ok_button_clicked,
              "on_newUserNameEntry_focus_out_event" : self.on_newUserNameEntry_focus_out_event,
              "on_homeDirCheck_toggled" : self.on_homeDirCheck_toggled,
              })

    #--function to reset the entry widgets in the user screen
    def userWinReset(self):
#        self.newUserNotebook.set_page(0)
        self.userWinUserName.grab_focus()
        self.userWinUserName.set_text("")
        self.userWinFullName.set_text("")
        self.userWinPassword.set_text("")
        self.userWinConfirm.set_text("")
        self.userWinHomeDir.set_text("")
        self.newGroupCheck.set_active(TRUE)

        default = '/bin/bash'
        if default in self.shells:
            self.loginShellCombo.list.select_item(self.shells.index(default))
#        self.groupChecklist.clear()

    def newUserWin(self, filter):
        self.filter = filter
        self.userWinReset()
#        self.fill_groups_list()
        self.userWin.show_all()


    #--------Event handlers for user window-----#
    def on_homeDirCheck_toggled(self, *args):
        self.homeDirHbox.set_sensitive(self.homeDirCheck.get_active())

    def on_newUserNameEntry_focus_out_event(self, *args):
        name = self.userWinUserName.get_text()
        self.userWinHomeDir.set_text("/home/%s" % name)

    def on_userWin_cancel_button_clicked(self, *args):
        self.userWinReset()
        self.userWin.hide()

    def on_userWin_ok_button_clicked(self, *args):
        userName = self.userWinUserName.get_text()
        if userName == "":
            dlg = GnomeMessageBox(_("Please enter a username."),
                                  MESSAGE_BOX_ERROR, STOCK_BUTTON_OK, None)
            dlg.run_and_close()
            return

        user = self.parent.ADMIN.lookupUserByName(userName)
        if user != None:
            dlg = GnomeMessageBox(_("A account with username '%s' already exists.") %userName,
                                  MESSAGE_BOX_ERROR, STOCK_BUTTON_OK, None)
            dlg.run_and_close()
            return

        pw = self.userWinPassword.get_text ()
        confirm = self.userWinConfirm.get_text ()

        if pw == confirm and len (pw) >= 6:
            pass
        else:
            if not pw and not confirm:
                dlg = GnomeMessageBox(_("Please enter password for the user."),
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK, None)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()
                return
            elif len (pw) < 6:
                dlg = GnomeMessageBox(_("Password is too short."),
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK, None)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()                
                return
            else:
                dlg = GnomeMessageBox(_("Passwords do not match."),
                                      MESSAGE_BOX_ERROR, STOCK_BUTTON_OK, None)
                dlg.run_and_close()
                self.userWinPassword.set_text("")
                self.userWinConfirm.set_text("")
                self.userWinPassword.grab_focus()                
                return

        userName = self.userWinUserName.get_text()
        fullName = self.userWinFullName.get_text()
        homeDir = self.userWinHomeDir.get_text()

        userEnt = self.parent.ADMIN.initUser(userName)
        userEnt.set(libuser.GECOS, [fullName])
        userEnt.set(libuser.HOMEDIRECTORY, [homeDir])

        uidNumber = userEnt.get(libuser.UIDNUMBER)[0]
        userEnt.set(libuser.LOGINSHELL, self.loginShellCombo.entry.get_text())

        if self.newGroupCheck.get_active():   #Create new group

            group = self.parent.ADMIN.lookupGroupByName(userName)
            if group != None:
                self.groupDialog.showDialog(userEnt, userName)

            else:
                groupEnt = self.parent.ADMIN.initGroup(userName)
                cn = groupEnt.get(libuser.GROUPNAME)[0]

                gidNumber = groupEnt.get(libuser.GIDNUMBER)[0]
                userEnt.set(libuser.GIDNUMBER, gidNumber)
                members = groupEnt.get(libuser.MEMBERUID)
                if not members:
                    members = []
                memberlist = string.join(members, ", ")

                self.parent.ADMIN.addGroup(groupEnt)
                self.parent.refresh_users_and_groups(cn)
                self.userWinReset()
                self.userWin.hide()

        else:            #Add user to group 'users'
            groupEnt = self.parent.ADMIN.lookupGroupByName('users')
            cn = groupEnt.get(libuser.GROUPNAME)[0]
            
            gidNumber = groupEnt.get(libuser.GIDNUMBER)[0]
            userEnt.set(libuser.GIDNUMBER, gidNumber)

            members = groupEnt.get(libuser.MEMBERUID)
            if not members:
                members = []

            members.append(userName)
            groupEnt.set(libuser.MEMBERUID, members)
            self.parent.ADMIN.modifyGroup(groupEnt)

            self.userWinReset()
            self.userWin.hide()

        if self.homeDirCheck.get_active():
            self.parent.ADMIN.addUser(userEnt, 1)
        else:
            self.parent.ADMIN.addUser(userEnt, 0)
            homeDir = ""

        self.parent.ADMIN.setpassUser(userEnt, pw)

        self.parent.refresh_users_and_groups([userName, 'users'])

##         self.userWinReset()
##         self.userWin.hide()

##     def toggle_row (self, row):
##         (val, row_data, name) = self.groupChecklist.get_row_data(row)
##         val = not val
##         self.groupChecklist.set_row_data(row, (val, row_data, name))
##         self.groupChecklist._update_row(row)

##         if self.groupChecklist.toggled_func != None:
##             self.groupChecklist.toggled_func(val)

##     def mouse_button_press(self, clist, event):
##         try:
##             row, col = self.groupChecklist.get_selection_info(event.x, event.y)
##         except:
##             pass
##         if row != None:
##             self.toggle_row(row)

##     def key_press_cb (self, clist, event):
##         if event.keyval == ord(" ") and self.groupChecklist.focus_row != -1:
##             self.toggle_row(self.groupChecklist.focus_row)

##     def fill_groups_list (self):
##         self.groups = parent.ADMIN.enumerateGroups()
##         row = 0

##         for group in self.groups:
##             row = self.groupChecklist.append_row((group, group), FALSE, group)
##             self.groupChecklist.set_row_data(row, (0, group, group))

